{{ dataframe }}

{% set let_line = 'let ' + table_id + ' = new simpleDatatables.DataTable("#' + table_id + '", {scrollY: "300px"});' %}
<script defer type="text/javascript">
    {{ let_line }}
</script>
