RX_CHANNELS = [37, 38, 39]
SET_SNIFFER = '!set_logger_mode 2'
CANCEL_SNIFFER = '!set_logger_mode 0'
CANCEL = '!cancel'
RESET_GW = '!reset'
VERSION = '!version'
SEND_ALL_ADV_CHANNELS = 0
SERIAL_TIMEOUT = 0.1
SEP = "#"*100
LENGTH = '1E'
GAP_TYPE = '16'
DEFAULT_ADVA = 'FFFFFFFFFFFF'
DEFAULT_DUPLICATES = 3
GW_APP_VERSION_HEADER = 'WILIOT_GW_BLE_CHIP_SW_VER'
DEFAULT_OUTPUT_POWER = 8
DEFAULT_DELAY = 15
COUPLING_DUPLICATIONS = [i for i in range(1, 6)]
COUPLING_TIME_DELAYS = [i for i in range(15, 256, 30)]
BRIDGES = [i for i in range(3)]
MAX_NFPKT = 65535
MAX_RSSI = 255
ADVA_LENGTH = 12
GAP_LENGTH = 4
SERVICE_UUID_LENGTH = 4
GROUP_ID_LENGTH = 6
UPLINK_DUPLICATIONS = [i for i in range(1, 6)]
UPLINK_TIME_DELAYS = [i for i in range(15, 256, 120)]

